@@ -5,7 +5,6 @@
 并行执行，
 统一返回结果
 '''
-import time
 import asyncio
 from concurrent.futures import ThreadPoolExecutor
 
@@ -21,7 +20,7 @@ class AsyncJobs(object):
         result = await self._loop.run_in_executor(None, self._work, arg)
         return result
 
-    def do(self, tasks, work, finish):
+    def do(self, tasks, work, finish=None):
         self._work = work
         works = [self._do_work(arg) for arg in tasks]
         jobs = asyncio.gather(*works)
@@ -35,24 +34,27 @@ class AsyncJobs(object):
 
 
 def test():
+    import time
     def hard_work(arg):
-        time.sleep(1)
-        return arg, arg*2
+        print(arg)
+        if arg==0:
+            time.sleep(5)
+        else:
+            time.sleep(1)
+        print(f'--{arg*2}--')
+        return arg*2
 
     def finish(results):
         print('%s'%results)
         pass
 
-    jobs = AsyncJobs(20)
-    while True:
-        aaa = time.time()
-        tasks = list(range(100))
-        jobs.do(tasks, hard_work, finish)
-        bbb = time.time()
-        print('%s' % (bbb - aaa))
+    jobs = AsyncJobs(8)
+    tasks = list(range(10))
+    aaa = time.time()
+    jobs.do(tasks, hard_work, finish)
+    bbb = time.time()
+    print('%s' % (bbb - aaa))
 
 
 if __name__ == '__main__':
     test()
-
-# 41.3
