@@ -63,7 +63,7 @@ class DBManager(object):
             keys = list(data[0].keys())
         else:
             keys = list(data.keys())
-        fields = ','.join(keys)  # 使用data的keys作为字段名
+        fields = '`'+'`,`'.join(keys)+'`'  # 使用data的keys作为字段名
         pre_values = ','.join([':%s' % k for k in keys])  # 前缀加":"
         if exist_action == ea_replace:
             pre_sql = 'replace into %s (%s) values(%s)' % (table, fields, pre_values)
