@@ -8,6 +8,7 @@ import time
 import traceback
 import sys
 import inspect
+import os, xlsxwriter
 
 
 def parse_arguments():
@@ -384,11 +385,93 @@ def print_start_end(logger=print, stacklevel=3):
     return decorator
 
 
+def make_excel(data, file_name, titles=[], widths=[], columns=[], other=[]):
+    '''
+    data: 数据
+    file_name: 文件名，比如`data.xlsx`
+    titles: 表头，['ID', '昵称', '账号', '消息内容']
+    widths: 列宽，[10, 20, 30, 60]
+    columns: 选取的列，如果不指定，按照data[i].values()顺序
+    titles/widths/columns要对应
+    other: 其他数据，指定行列和值
+        [
+          {'row': 0, 'col': 0, 'value': '数值'}, # 行列从0开始
+        ]
+    '''
+    path = os.path.dirname(file_name)
+    if path and not os.path.exists(path):
+        os.makedirs(path) # 创建多级目录
+
+    rows = []
+    cell_fmt = 'A%d'
+    if titles:
+        rows.append(titles)
+    if not columns:
+        # 未指定列，按顺序填充表格
+        for it in data:
+            row = list(it.values())
+            rows.append(row)
+    else:
+        # 指定列，按指定次序填充表格
+        for it in data:
+            row = []
+            for k in columns:
+                row.append(it[k])
+            rows.append(row)
+
+    workbook = xlsxwriter.Workbook(file_name)
+    worksheet = workbook.add_worksheet()
+
+    for i in range(len(rows)):
+        row = rows[i]
+        cell = cell_fmt % (i+1)
+        worksheet.write_row(cell, row)
+
+    for it in other:
+        row = it['row']
+        col = it['col']
+        value = it['value']
+        worksheet.write_row(row, col, [value])
+
+    if widths:
+        for i in range(len(widths)):
+            worksheet.set_column(i,i,widths[i])
+    workbook.close()
+
+
+def save_excel(path, data, file_name, titles=[], widths=[], columns=[]):
+    file_path = os.path.join(path, file_name)
+    make_excel(data, file_path, titles, widths, columns)
+    return file_path
+
+
+def reg_find_all(pattern, text):
+    '''
+    正则查找匹配到的结果
+    正则表达式可以包含分组，
+    会拆解分组，只获取匹配到的关键词
+    例如： pattern = '(?:^|[^0-9a-zA-Z])(123)(?:$|[^0-9a-zA-Z])|(aa|bb)'
+    是匹配`123|aa|bb`三个关键词。当然需要自己设定哪些分组是要忽略的，分组内加前缀`?:`
+    '''
+    result = re.findall(pattern, text)
+    if not result:
+        return []
+    found = []
+    for r in result:
+        if type(r) == tuple:
+            found.append(''.join(r))
+        else:
+            found.append(r)
+    return found
+
 
 if __name__ == '__main__':
-    @print_start_end()
-    def main():
-        result = parse_arguments()
-        print(result)
-        pass
-    main()
+    # def test():
+    #     data = [
+    #         {'id':1, 'name':'tao', 'msg': 'hello'},
+    #         {'id':2, 'name':'xia', 'msg': 'world'},
+    #         {'id':3, 'name':'mao', 'msg': 'hi'},
+    #     ]
+    #     save_excel('.', data, 'test.xlsx', ['NAME', 'MSG'], [20, 30], ['name', 'msg'])
+    # test()
+    pass
